# encoding: utf-8

# File:	clients/ncs_auto.ycp
# Package:	Configuration of Cluster Services (NCS)
# Summary:	Client for autoinstallation
# Authors:	Michael G. Fritch <mgfritch@novell.com>
#
# $Id: ncs_auto.ycp,v 1.1.2.4 2005/04/29 22:13:15 rhaslam Exp $
#
# This is a client for autoinstallation. It takes its arguments,
# goes through the configuration and return the setting.
# Does not do any changes to the configuration.

# @param function to execute
# @param map/list of ncs settings
# @return [Hash] edited settings, Summary or boolean on success depending on called function
# @example map mm = $[ "FAIL_DELAY" : "77" ];
# @example map ret = WFM::CallFunction ("ncs_auto", [ "Summary", mm ]);
module Yast
  class NcsAutoClient < Client
    def main
      Yast.import "UI"

      textdomain "novell-ncs"

      Builtins.y2milestone("----------------------------------------")
      Yast.import "Ncs"
      Yast.include self, "novell-ncs/wizards.rb"

      @ret = nil
      @func = ""
      @param = {}

      # Check arguments
      if Ops.greater_than(Builtins.size(WFM.Args), 0) &&
          Ops.is_string?(WFM.Args(0))
        @func = Convert.to_string(WFM.Args(0))
        if Ops.greater_than(Builtins.size(WFM.Args), 1) &&
            Ops.is_map?(WFM.Args(1))
          @param = Convert.to_map(WFM.Args(1))
        end
      end
      Builtins.y2milestone("Ncs auto started. Func = %1", @func)
      Builtins.y2debug("func=%1", @func)
      Builtins.y2debug("param=%1", @param)

      # Create a summary
      if @func == "Summary"
        Ncs.autoyast_tool = true
        @ret = Ops.get_string(Ncs.Summary, 0, "")
      # Reset configuration
      elsif @func == "Reset"
        Ncs.Import({})
        @ret = {}
      # Change configuration (run AutoSequence)
      elsif @func == "Change"
        Ncs.autoyast_tool = true
        #    import "OesLdap";
        #    OesLdap::autoyast_tool = true;
        @ret = NcsAutoSequence()
      # Import configuration
      elsif @func == "Import"
        @ret = Ncs.Import(@param)
      # Return actual state
      elsif @func == "Export"
        Ncs.Read if Ncs.Settings == nil || Ncs.Settings == {}
        Ncs.auto_clone = true
        @ret = Ncs.Export
      # Return needed packages
      elsif @func == "Packages"
        @ret = Ncs.AutoPackages
      # Read current state
      elsif @func == "Read"
        Yast.import "Progress"
        @progress_orig = Progress.set(false)
        @ret = Ncs.Read
        Progress.set(@progress_orig)
      # Write givven settings
      elsif @func == "Write"
        Yast.import "Progress"
        @progress_orig = Progress.set(false)
        Ncs.write_only = true
        @ret = Ncs.Write
        Progress.set(@progress_orig)
      # Return if configuration  was changed
      # return boolean
      elsif @func == "GetModified"
        Ncs.globals_configured = true
        @ret = Ncs.globals_configured
      # Set modified flag
      # return boolean
      elsif @func == "SetModified"
        Ncs.globals_configured = true
        @ret = true
      else
        Builtins.y2error("Unknown function: %1", @func)
        @ret = false
      end

      Builtins.y2debug("ret=%1", @ret)
      Builtins.y2milestone("Ncs auto finished")
      Builtins.y2milestone("----------------------------------------")

      deep_copy(@ret) 

      # EOF
    end
  end
end

Yast::NcsAutoClient.new.main
