# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:	clients/ncs_upgrade.ycp
# Package:	Configuration of NCS
# Summary:	upgrade function dispatcher.
# Authors:	Mark Clark <mclark@novell.com>
#
# $Id: ncs_upgrade.ycp,v 1.1.1.10 2007/04/02 21:20:32 mclark Exp $
#
# upgrade function dispatcher for NCS configuration.
module Yast
  class NcsUpgradeClient < Client
    def main

      textdomain "novell-ncs"

      Yast.import "NovellAddonInstall"

      # The main ()
      Builtins.y2milestone("----------------------------------------")
      Builtins.y2milestone("NCS upgrade started")

      @func = Convert.to_string(WFM.Args(0))
      @param = Convert.to_map(WFM.Args(1))
      @ret = {}

      # return the status of this product
      if @func == "Info"
        @cfg_file = "/etc/sysconfig/NCS" # old config file path
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Info_oes2_fcs"
        @cfg_file = "/etc/sysconfig/novell/ncs" # old OES 2 FCS config file path
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Info_oes2"
        @cfg_file = "/etc/sysconfig/novell/ncs1" # old OES 2 SP1 config file path
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Info_oes2_sp2"
        @cfg_file = "/etc/sysconfig/novell/ncs2_sp2" # old OES 2 SP1 config file path
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Info_oes2_sp3"
        @cfg_file = "/etc/sysconfig/novell/ncs2_sp3" # old OES 2 SP1 config file path
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Info_oes11"
        @cfg_file = "/etc/sysconfig/novell/ncs2_oes11"
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Info_oes11_sp1"
        @cfg_file = "/etc/sysconfig/novell/ncs_oes11_sp1"
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Info_oes11_sp2"
        @cfg_file = "/etc/sysconfig/novell/ncs_oes11_sp2"
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Info_oes11_sp3"
        @cfg_file = "/etc/sysconfig/novell/ncs_oes11_sp3"
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Info_oes2015"
        @cfg_file = "/etc/sysconfig/novell/ncs_oes2015"
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Info_oes2015_sp1"
        @cfg_file = "/etc/sysconfig/novell/ncs_oes2015_sp1"
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Info_oes2018"
        @cfg_file = "/etc/sysconfig/novell/ncs_oes2018"
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Info_oes2018_sp1"
        @cfg_file = "/etc/sysconfig/novell/ncs_oes2018_sp1"
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Info_oes2018_sp2"
        @cfg_file = "/etc/sysconfig/novell/ncs_oes2018_sp2"
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      elsif @func == "Translate"
        @cfg_file = "/etc/sysconfig/novell/ncs_oes2018_sp3"
        @ret = NovellAddonInstall.ReadConfigFileSettings(@cfg_file)
      else
        Builtins.y2error("unknown function: %1", @func)
      end

      # Finish
      Builtins.y2debug("ret=%1", @ret)
      Builtins.y2milestone("NCS upgrade finished")
      Builtins.y2milestone("----------------------------------------")
      deep_copy(@ret) 

      # EOF
    end
  end
end

Yast::NcsUpgradeClient.new.main
