# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:	include/novell-ncs/dialogs.ycp
# Package:	Configuration of Cluster Services (NCS)
# Summary:	Dialogs definitions
# Authors:	Michael G. Fritch <mgfritch@novell.com>
#
# $Id: dialogs.ycp,v 1.1.2.39 2005/11/29 02:35:06 rjorgensen Exp $
module Yast
  module NovellNcsDialogsInclude
    def initialize_novell_ncs_dialogs(include_target)
      Yast.import "UI"

      textdomain "novell-ncs"

      Yast.import "Label"
      Yast.import "Wizard"
      Yast.import "Report"
      Yast.import "Popup"
      Yast.import "Mode"
      Yast.import "Stage"
      Yast.import "String"
      Yast.import "Ncs"
      Yast.import "NovellValidators"
      Yast.import "OesLdap"
      Yast.import "NovellLDAP"
      Yast.import "NovellLdapBrowser"
      Yast.import "NovellLDAPUI"
      Yast.import "NovellUtils"
      Yast.import "EdirUtils"
      Yast.include include_target, "novell-ncs/helps.rb"
      Yast.include include_target, "novell-ncs/complex.rb"
      Yast.include include_target, "novell-ncs/routines.rb"

      @goForwardAnyway = true
    end

    def PromptIfNeededForEdirAdminPassword
      useXad = false

      if Ncs.xadMode
        xad_tree_admin_context = Ops.get_string(
          NovellLDAP.Settings,
          "XAD_TREE_ADMIN_CONTEXT",
          ""
        )
        if xad_tree_admin_context != nil && xad_tree_admin_context != ""
          useXad = true
        end # if this value is set at all, then it is the eDir admin (in spite of xad name)
      end
      Builtins.y2milestone(
        "Ncs.PromptIfNeededForEdirAdminPassword:use xad:%1",
        useXad
      )

      if useXad
        if Ops.get_string(NovellLDAP.Settings, "XAD_TREE_ADMIN_PASSWORD", "") == ""
          NovellLDAPUI.XadTreeAdminPasswordDialog
        end
      else
        if Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_PASSWORD", "") == ""
          NovellLDAPUI.AdminPasswordDialog
        end
      end
      Builtins.y2milestone(
        "Ncs.PromptIfNeededForEdirAdminPassword:useXad %1",
        useXad
      )
      useXad
    end


    # Validate that no one is using the cluster ip address
    # @return true if unsused
    def ValidateClusterIP(clusterIP)
      cmd = Builtins.sformat("ping -w 15 -c 1 %1", clusterIP)
      rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      return false if Ops.get_integer(rmap, "exit", -1) == 0
      true
    end






    # Query LDAP to see if this is a shared disk cluster
    # @return true if there is a shared disk
    def isSharedDiskCluster(admin_dn, passwd, ldapServer, securePort, object)
      @goForwardAnyway = true

      admin = NovellUtils.Nds2Ldap(admin_dn)
      ipaddress = ldapServer
      context = NovellUtils.Nds2Ldap(NovellUtils.GetContextNDS(object))
      port = securePort
      index = Builtins.search(object, ".")
      cn = Builtins.substring(object, 0, index)

      logcmd = Builtins.sformat(
        "/usr/bin/ldapsearch -H ldaps://%1:%2 -D '%3' -w '%4' -x -s one -LLL -b '%5' '%6' nCSSharedDiskFlag",
        ipaddress,
        port,
        admin,
        "****",
        context,
        cn
      )

      cmd = Builtins.sformat(
        "/usr/bin/ldapsearch -H ldaps://%1:%2 -D '%3' -w '%4' -x -s one -LLL -b '%5' '%6' nCSSharedDiskFlag",
        ipaddress,
        port,
        admin,
        passwd,
        context,
        cn
      )

      Builtins.y2milestone("Checking shared disk flag %1", object)
      Builtins.y2milestone("Executing command %1", logcmd)
      rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      if Ops.get_integer(rmap, "exit", -1) != 0
        Builtins.y2error(
          "Execution of ldap_search failed, error returned = %1",
          Ops.get_integer(rmap, "exit", -1)
        )
      end

      if Ops.get_string(rmap, "stderr", "failed") != ""
        if false ==
            Popup.YesNoHeadline(
              _("LDAP Search Failure!"),
              Builtins.sformat(
                _(
                  "The ldapsearch command for %1 failed. The context may be invalid. %2\nContinue Anyway?"
                ),
                object,
                Ops.get_string(rmap, "stderr", "")
              )
            )
          @goForwardAnyway = false
        end
        return true
      end
      output = Ops.get_string(rmap, "stdout", "Bogus output")
      Builtins.y2milestone("stdout from command = %1", output)
      Builtins.y2milestone("Checking object shared disk flag")
      testString = "nCSSharedDiskFlag: TRUE"
      index = Builtins.search(output, testString)
      if index == -1
        return false # object not found OR not TRUE
      else
        return true # object found and value TRUE
      end
    end

    def CleanUpString(str)
      return "" if str == nil || str == ""

      stripChars = " \n\t "

      # Get rid of junk on the end of the string
      str = NovellUtils.StripTrailing(str, stripChars)

      # Get rid of junk in the middle of the string (bugzilla 484835)
      str = String.Replace(str, "\n", "") # kill embedded line feed

      # Can't have this since a context may have a space in it.
      #    str = String::Replace( str, " ", "");   // kill embedded space

      str
    end

    # Get actual directory object dn using ldapsearch.
    #
    # @param [String] admin_dn string admin FDN
    # @param [String] passwd string admin password
    # @param [String] ldapServer string ldap server address
    # @param [Fixnum] securePort integer LDAPS port
    # @param object string object we are preforming the ldapsearch for
    # @param [String] objectClass string objectClass of the object we are using ldapsearch for
    #
    # @return [String] or "" if object was not found
    def GetDirectoryObjectsActualDN(admin_dn, passwd, ldapServer, securePort, objectDN, objectClass)
      admin = NovellUtils.Nds2Ldap(admin_dn)
      ipaddress = ldapServer
      port = securePort
      checkClass = true

      # if object class is empty string; turn off checkClass
      checkClass = false if objectClass == ""
      admin = NovellUtils.Nds2Ldap(admin) if !EdirUtils.isLDAPdn(admin)

      # Formulate the ldapsearch command.
      # -l 30 (allow 15 seconds for completion)
      # -H specify the URI (IPaddress and Port)
      # -D user to login as
      # -x use simple authentication
      # -y password in file for user
      # -w password for user - replace -y for testing
      # -s how deep to search from starting container
      # -LLL filter output to get just the results in a simple format
      # -b container in which to start the search
      # followed by the thing to search for

      logcmd = ""
      cmd = ""

      if checkClass
        logcmd = Builtins.sformat(
          "/usr/bin/ldapsearch -l 30 -H ldaps://%1:%2 -D '%3' -w '%4' -x -s base -LLL -b '%5' objectClass=%6 dn",
          ipaddress,
          port,
          admin,
          "*****",
          objectDN,
          objectClass
        )

        cmd = Builtins.sformat(
          "/usr/bin/ldapsearch -l 30 -H ldaps://%1:%2 -D '%3' -w '%4' -x -s base -LLL -b '%5' objectClass=%6 dn",
          ipaddress,
          port,
          admin,
          passwd,
          objectDN,
          objectClass
        )
      else
        logcmd = Builtins.sformat(
          "/usr/bin/ldapsearch -l 30 -H ldaps://%1:%2 -D '%3' -w '%4' -x -s base -LLL -b '%5' dn",
          ipaddress,
          port,
          admin,
          "*****",
          objectDN
        )

        cmd = Builtins.sformat(
          "/usr/bin/ldapsearch -l 30 -H ldaps://%1:%2 -D '%3' -w '%4' -x -s base -LLL -b '%5' dn",
          ipaddress,
          port,
          admin,
          passwd,
          objectDN
        )
      end

      Builtins.y2milestone(
        "Checking existance of object %1 of type %2",
        objectDN,
        objectClass
      )
      Builtins.y2milestone("Executing command %1", logcmd)
      rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      if Ops.get_integer(rmap, "exit", -1) != 0
        Builtins.y2milestone(
          "Execution of ldap_search for object %1 failed, error returned = %2\n%3",
          objectDN,
          Ops.get_integer(rmap, "exit", -1),
          rmap
        )
        return ""
      end

      if Ops.get_string(rmap, "stdout", "failed") == nil ||
          Ops.get_string(rmap, "stdout", "failed") == ""
        Builtins.y2milestone(
          "The call to find object %1 failed.\n%2",
          objectDN,
          rmap
        )
        return ""
      end

      rmap_stdout = Ops.get_string(rmap, "stdout", "")
      idx = Builtins.search(rmap_stdout, "\n")
      if idx != nil # Only look on the first line of the output.
        rmap_stdout = Builtins.substring(rmap_stdout, 0, idx)
      end
      # Make sure we have not found a bogus object such as a comment
      if Builtins.regexpmatch(rmap_stdout, "^[ \t]*#")
        Builtins.y2milestone(
          "The the call to find object %1 failed. Comment found in return:\n%2",
          objectDN,
          Ops.get_string(rmap, "stdout", "")
        )
        return ""
      end

      # We have found an object of correct objectClass!!
      output = Ops.get_string(rmap, "stdout", "Bogus output")
      Builtins.y2milestone("stdout from command:\n%1", output)

      output = NovellUtils.CombineLdapSearchReturn(output) # bugzilla 630237

      # output example
      # dn: cn=kcb_cluster,o=novell
      idx = 4 # skip dn:_
      realDN = Builtins.substring(output, idx)
      realDN = CleanUpString(realDN)
      Builtins.y2milestone(
        "Ncs.GetDirectoryObjectsActualDN:returning realDN=%1",
        realDN
      )
      realDN
    end

    # Check to see if we can find a SBD for this cluster
    # @return true if we can find it.
    def FindSBD(clusterFDN)
      extractedFDN = NovellUtils.ExtractCN(NovellUtils.Nds2Ldap(clusterFDN))
      clusterCN = Ops.get_string(extractedFDN, 0, "unnamed")
      cmd = Builtins.sformat(
        "/opt/novell/ncs/bin/sbdutil -f -s -n '%1'",
        clusterCN
      )
      rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      return true if Ops.get_integer(rmap, "exit", -1) == 0
      false
    end


    # Get a list of device that can accept a new SBD partition
    def DeviceList
      rmap = Convert.to_map(
        SCR.Execute(
          path(".target.bash_output"),
          "/opt/novell/ncs/bin/sbdutil -l"
        )
      )
      Builtins.sort(
        Builtins.splitstring(Ops.get_string(rmap, "stdout", ""), "\n")
      )
    end

    def isNumeric(val)
      retCode = false
      numbers = "0123456789"

      retCode = true if Builtins.findfirstnotof(val, numbers) == nil

      retCode
    end

    # NewOrExisting dialog
    # @return dialog result
    def NewOrExistingDialog
      if Convert.to_string(SCR.Read(path(".sysconfig.ncs.SERVICE_CONFIGURED"))) == "yes"
        Popup.Message(
          _("NCS is already configured, you cannot reconfigure it.")
        )
        Builtins.y2milestone(
          "NCS is already configured, you cannot reconfigure it."
        )
        return :skip
      end
      if  Ncs.upgradeModeOES11SP2 ||
          Ncs.upgradeModeOES11SP3 ||
          Ncs.upgradeModeOES2015 ||
          Ncs.upgradeModeOES2015SP1 ||
          Ncs.upgradeModeOES2018 ||
          Ncs.upgradeModeOES2018SP1 ||
          Ncs.upgradeModeOES2018SP2 
        Popup.Message(NovellUtils.noConfigMessage)
        Builtins.y2milestone(
          "Ncs.NewIrExistingDialog:skipping GUI since we are in upgrade mode."
        )
        return :skip
      end

      #string serverDn = NovellUtils::Nds2Ldap(GetServerFDN());
      #y2error("serverDn =============================> \'%1\'", serverDn);

      # Ncs NewOrExisting dialog caption
      caption = _("OES Cluster Services (NCS) Configuration")

      # Ncs NewOrExisting dialog contents
      ldap_servers = Ops.get_list(Ncs.Settings, "CONFIG_NCS_LDAP_SERVERS", [])
      ldap_servers = Builtins.filter(ldap_servers) do |address|
        NovellLDAP.AddressExists(address, nil)
      end

      local_server_list = NovellLDAP.GetLocalServers(nil)
      idx = 0
      done = false
      Builtins.foreach(ldap_servers) do |ldap_server|
        Builtins.foreach(local_server_list) do |local_server|
          Builtins.y2milestone(
            "Ncs.NewOrExistingDialog:ldap server is %1, local server is %2",
            ldap_server,
            Ops.get_string(local_server, "CONFIG_LDAP_SERVER_IP", "")
          )
          if ldap_server ==
              Ops.get_string(local_server, "CONFIG_LDAP_SERVER_IP", "")
            done = true
            raise Break
          end
        end
        if done
          raise Break
        else
          idx = Ops.add(idx, 1)
        end
      end

      clusterFDN = Ops.get_string(Ncs.Settings, "CONFIG_NCS_CLUSTER_DN", "")
      clusterIP = Ops.get_string(Ncs.Settings, "CONFIG_NCS_CLUSTER_IP", "")
      device = Ops.get_string(Ncs.Settings, "CONFIG_NCS_SBD_DEV", "")
      device2 = Ops.get_string(Ncs.Settings, "CONFIG_NCS_SBD_DEV2", "")
      devices = DeviceList()
      server_list = Ops.get_list(
        NovellLDAP.Settings,
        "CONFIG_LDAP_SERVERS_LIST",
        []
      )
      deviceSize = Ops.get_string(Ncs.Settings, "CONFIG_NCS_SBD_SIZE", "8")

      server_list_widget = NovellLDAPUI.ServerListWidget(
        server_list,
        _("Directory Server Address"),
        Ops.get(ldap_servers, idx, ""),
        true,
        :multi
      )
      ldap_widget = Ops.get_term(server_list_widget, "term") { VBox(:Empty) }

      useMaxSize = true
      useMax = Ops.get_string(Ncs.Settings, "CONFIG_NCS_USE_MAX_SIZE", "no")
      useMaxSize = false if useMax != "yes"
      device2 = devices
      contents = HVSquash(
        VBox(
          Left(Label(_("New or Existing Cluster"))),
          RadioButtonGroup(
            Id(:mode),
            Opt(:notify),
            VBox(
              Left(
                RadioButton(
                  Id(:new),
                  Opt(:notify),
                  _("Ne&w Cluster"),
                  Ops.get_string(
                    Ncs.Settings,
                    "CONFIG_NCS_CONFIG_TYPE",
                    "New Cluster"
                  ) == "New Cluster"
                )
              ),
              Left(
                RadioButton(
                  Id(:existing),
                  Opt(:notify),
                  _("E&xisting Cluster"),
                  Ops.get_string(Ncs.Settings, "CONFIG_NCS_CONFIG_TYPE", "") == "Existing Cluster"
                )
              )
            )
          ),
          ldap_widget,
          MinWidth(
            45,
            HBox(
              TextEntry(
                Id(:clusterFDN),
                _("Cluster &FDN (e.g. cn=cluster,o=microfocus)"),
                clusterFDN
              ),
              VBox(Label(""), PushButton(Id(:browse), _("Browse")))
            )
          ),
          TextEntry(Id(:clusterIP), _("Cluster &IP address"), clusterIP),
          Left(
          ComboBox(
            Id(:device),
             Opt(:notify, :immediate, :hstretch),
            _("Select the storage &device with shared media"),
            devices
          )),
          Left(
            ReplacePoint(
            Id(:list_device2), 
            ComboBox(Id(:device2), Opt(:notify,:hstretch),  
            _(
              "Select the device where the &mirror partition will be created (optional)"
            ),
            devices
          ))),
          MinWidth(
            45,
            HBox(
              TextEntry(
                Id(:deviceSize),
                _("Desired partition size of shared media (MB)"),
                deviceSize
              ),
              VBox(
                Label(""),
                Left(
                  CheckBox(
                    Id(:maxSize),
                    Opt(:notify),
                    _("&Use maximum size"),
                    useMaxSize
                  )
                )
              )
            )
          )
        )
      )

      helps = Ops.get_string(@HELPS, "NewOrExisting", "")
      Wizard.SetContentsButtons(
        caption,
        contents,
        helps,
        Label.BackButton,
        Label.NextButton
      )

      # Make Existing Cluster the default
      if Ops.get_string(Ncs.Settings, "CONFIG_NCS_CONFIG_TYPE", "") == ""
        UI.ChangeWidget(Id(:mode), :CurrentButton, :existing)
      elsif Ops.get_string(Ncs.Settings, "CONFIG_NCS_CONFIG_TYPE", "") == "New Cluster"
        UI.ChangeWidget(Id(:mode), :CurrentButton, :new)
      elsif Ops.get_string(Ncs.Settings, "CONFIG_NCS_CONFIG_TYPE", "") == "Existing Cluster"
        UI.ChangeWidget(Id(:mode), :CurrentButton, :existing)
      end

      UI.ChangeWidget(Id(:device), :Value, device)
      UI.ChangeWidget(Id(:device2), :Value, device2)
      UI.ChangeWidget(Id(:browse), :Enabled, false)

      isldaprunning = 1
      primary_server = NovellLDAP.BrowseButtonLdapServer(nil)
      cmd =  Builtins.sformat("/usr/bin/ldapsearch -x -H ldaps://%1:636 -s base -b \"\" > /dev/null",
          Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", ""))

      isldaprunning = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))

      if Convert.to_symbol(UI.QueryWidget(Id(:mode), :CurrentButton)) == :existing
        if !(NovellLDAP.IsLocalLDAPRunning == 0 || isldaprunning == 0)
          UI.ChangeWidget(Id(:browse), :Enabled, false)
        else
          UI.ChangeWidget(Id(:browse), :Enabled, true)
        end

        UI.ChangeWidget(Id(:clusterIP), :Enabled, false)
        UI.ChangeWidget(Id(:device), :Enabled, false)
        UI.ChangeWidget(Id(:device2), :Enabled, false)
        UI.ChangeWidget(Id(:clusterFDN), :Enabled, true)
        UI.ChangeWidget(Id(:deviceSize), :Enabled, false)
        UI.ChangeWidget(Id(:maxSize), :Enabled, false)
      elsif Convert.to_symbol(UI.QueryWidget(Id(:mode), :CurrentButton)) == :new
        if !(NovellLDAP.IsLocalLDAPRunning == 0 || isldaprunning == 0)
          UI.ChangeWidget(Id(:browse), :Enabled, false)
        else
          UI.ChangeWidget(Id(:browse), :Enabled, true)
        end

        UI.ChangeWidget(Id(:clusterIP), :Enabled, true)
        UI.ChangeWidget(Id(:device), :Enabled, true)
        UI.ChangeWidget(Id(:device2), :Enabled, false)
        UI.ChangeWidget(Id(:clusterFDN), :Enabled, true)

        if useMaxSize
          UI.ChangeWidget(Id(:deviceSize), :Enabled, false)
        else
          UI.ChangeWidget(Id(:deviceSize), :Enabled, true)
        end

        UI.ChangeWidget(Id(:maxSize), :Enabled, true)
      end

      PromptIfNeededForEdirAdminPassword()

      ret = nil
      while true

        ret = UI.UserInput
        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :device
#          Code to generate the 4k or 512 device list
          device = Convert.to_string(UI.QueryWidget(Id(:device), :Value))
          devices_map = {}
    
#          Generate the map, devicename:size
          Builtins.foreach(devices) do |dev|
            cmd = "nlvm list device #{dev} | grep \-o \"SectorSize(logical/physical)\=.*b\/\" | tr \-dc \'0-9\'"
            rmap = Convert.to_map(
              SCR.Execute(
                path(".target.bash_output"),cmd)
            )
            sector_size= NovellUtils.CleanUpString(Ops.get_string(rmap, "stdout", ""))
            Ops.set(devices_map,dev,sector_size)
          end
          get_selected_device_sector_size=Ops.get(devices_map,device)
          deviceSize="8" if get_selected_device_sector_size == "512" || get_selected_device_sector_size == ""
          deviceSize="64" if get_selected_device_sector_size == "4096"

#          Code to filter the devices
          device2=[]
          device2 = Map.Keys(Builtins.filter(devices_map){ |key,value| value == get_selected_device_sector_size})
          device2 = Builtins.filter(device2) { |f| f != device}
          empty_device=[" "]
          device2 = Builtins.merge(empty_device,device2)
        #  UI.ChangeWidget(Id(:device2), :Enabled, true)
          UI.ChangeWidget(Id(:deviceSize), :Value, deviceSize)
          Ops.set(Ncs.Settings, "CONFIG_NCS_SBD_SIZE", deviceSize)
         
          UI.ReplaceWidget(
            Id(:list_device2),
            ComboBox(Id(:device2), Opt(:notify,:hstretch),
            _(
              "Select the device where the &mirror partition will be created (optional)"
            ),
            device2
          ))
          next
        elsif ret == :next
          rb = Convert.to_symbol(UI.QueryWidget(Id(:mode), :CurrentButton))
          ldap_servers = Convert.convert(
            UI.QueryWidget(Id(:server_select), :SelectedItems),
            :from => "any",
            :to   => "list <string>"
          )
          primary_server2 = NovellLDAP.GetAddressServer(
            Ops.get(ldap_servers, 0, ""),
            server_list
          )

          # These are internal work variables used in another dialog.
          Ops.set(
            Ncs.Settings,
            "PRIMARY_IP",
            Ops.get_string(primary_server2, "CONFIG_LDAP_SERVER_IP", "")
          )
          Ops.set(
            Ncs.Settings,
            "SECURE_LDAP",
            Ops.get_integer(primary_server2, "CONFIG_LDAP_SECURE_PORT", 636)
          )

          clusterFDN = Convert.to_string(
            UI.QueryWidget(Id(:clusterFDN), :Value)
          )
          clusterIP = Convert.to_string(UI.QueryWidget(Id(:clusterIP), :Value))
          device = Convert.to_string(UI.QueryWidget(Id(:device), :Value))

          Builtins.y2milestone("NCS:device is '%1'", device)
          wrkDevice = NovellUtils.CleanUpString(device)
          Builtins.y2milestone("NCS:wrkDevice is '%1'", wrkDevice)
          if rb == :new && (wrkDevice == nil || wrkDevice == "")
            erMsg = _(
              "Warning:You have not selected a shared media. The cluster SBD partition will not be created.\nContinue anyway?"
            )
            next if Popup.ContinueCancel(erMsg) == false
          end

          device2 = Convert.to_string(UI.QueryWidget(Id(:device2), :Value))

          useMaxSize = Convert.to_boolean(UI.QueryWidget(Id(:maxSize), :Value))
          if useMaxSize
            deviceSize = "-1"
          else
            deviceSize = Convert.to_string(
              UI.QueryWidget(Id(:deviceSize), :Value)
            )
          end

          adminDn = Ops.get_string(
            NovellLDAP.Settings,
            "CONFIG_LDAP_ADMIN_CONTEXT",
            ""
          )
          adminPassword = Ops.get_string(
            NovellLDAP.Settings,
            "CONFIG_LDAP_ADMIN_PASSWORD",
            ""
          )
          if Ncs.xadMode
            xad_tree_admin_context = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_CONTEXT",
              ""
            )
            Builtins.y2milestone(
              "NovellLum:XAD_TREE_ADMIN_CONTEXT:%1",
              xad_tree_admin_context
            )
            if xad_tree_admin_context != nil && xad_tree_admin_context != ""
              adminDn = NovellUtils.Nds2Ldap(xad_tree_admin_context)
              adminPassword = Ops.get_string(
                NovellLDAP.Settings,
                "XAD_TREE_ADMIN_PASSWORD",
                ""
              )
            end
          end
          Builtins.y2milestone("Ncs:admin:%1", adminDn)

          # Validate the device size
          if rb == :new
            if !useMaxSize
              # Make sure we have a value
              if deviceSize == nil || deviceSize == ""
                Popup.Error(_("A partition size must be entered."))
                UI.SetFocus(Id(:deviceSize))
                next
              end
              # Make sure the value is numeric.
              if !isNumeric(deviceSize)
                Popup.Error(_("Partition size must be a numeric value."))
                UI.SetFocus(Id(:deviceSize))
                next
              end
              # Make sure the value is greater than 8.
              if Ops.less_than(
                  Builtins.tofloat(deviceSize),
                  Convert.convert(8, :from => "integer", :to => "float")
                )
                Popup.Error(_("Partition size must be 8 MB or greater."))
                UI.SetFocus(Id(:deviceSize))
                next
              end
            end
          end

          # make sure all fields have data
          if ldap_servers == nil || ldap_servers == []
            Popup.Error(_("At least one Directory Server must be selected."))
            UI.SetFocus(Id(:server_select))
            next
          end

          if clusterFDN == "" || rb == :new && clusterIP == ""
            Popup.Error(_("All fields must be filled in."))
            next
          end

          # Make sure we have specified a container.
          if clusterFDN != "" && Builtins.findfirstof(clusterFDN, ".,") == nil
            Popup.Error(
              _("A container must be specified (e.g. cn=cluster,o=microfocus).")
            )
            next if Ncs.testMode == false
          end

          # Validate the FDN
          if !NovellValidators.ValidateFDN(clusterFDN, false)
            next if Ncs.testMode == false
          end
          clusterFDN = NovellUtils.Nds2Ldap(clusterFDN)

          newCluster = false
          newCluster = true if rb == :new

          if !Ncs.autoyast_tool && !(Mode.installation && !Stage.normal)
            serverIsDsfw = Ncs.ServerIsDsfwServer(
              Ops.get_string(primary_server2, "CONFIG_LDAP_SERVER_IP", "")
            )

            ldapSecurePort = Ops.get_integer(
              primary_server2,
              "CONFIG_LDAP_SECURE_PORT",
              636
            )
            ldapSecurePort = 1636 if serverIsDsfw

            if newCluster
              # Validate absence of the cluster object in the directory
              exists = EdirUtils.DoesDirectoryObjectExist(
                adminDn,
                adminPassword,
                Ops.get_string(primary_server2, "CONFIG_LDAP_SERVER_IP", ""),
                ldapSecurePort,
                clusterFDN,
                ""
              )
              if exists
                Report.Error(
                  Builtins.sformat(
                    _("The cluster object %1 already exists in the directory"),
                    clusterFDN
                  )
                )
                next if Ncs.testMode == false
              elsif ldapSecurePort == 1636 # try again just in case we must use port 636
                exists2 = EdirUtils.DoesDirectoryObjectExist(
                  adminDn,
                  adminPassword,
                  Ops.get_string(primary_server2, "CONFIG_LDAP_SERVER_IP", ""),
                  636,
                  clusterFDN,
                  ""
                )
                if exists2
                  Report.Error(
                    Builtins.sformat(
                      _("The cluster object %1 already exists in the directory"),
                      clusterFDN
                    )
                  )
                  next if Ncs.testMode == false
                end
              end
              extractedFDN = NovellUtils.ExtractCN(
                NovellUtils.Nds2Ldap(clusterFDN)
              )
              clusterCont = NovellUtils.Ldap2Nds(
                Ops.get_string(extractedFDN, 1, "unnamed")
              )

              exists = EdirUtils.DoesDirectoryObjectExist(
                adminDn,
                adminPassword,
                Ops.get_string(primary_server2, "CONFIG_LDAP_SERVER_IP", ""),
                ldapSecurePort,
                clusterCont,
                ""
              )
              if !exists
                if ldapSecurePort == 1636 # try again just in case we must use port 636
                  exists2 = EdirUtils.DoesDirectoryObjectExist(
                    adminDn,
                    adminPassword,
                    Ops.get_string(primary_server2, "CONFIG_LDAP_SERVER_IP", ""),
                    636,
                    clusterCont,
                    ""
                  )
                  if !exists2
                    Report.Error(
                      Builtins.sformat(
                        _("The container %1 does not exist in the directory"),
                        clusterCont
                      )
                    )
                    next if Ncs.testMode == false
                  end
                else
                  Report.Error(
                    Builtins.sformat(
                      _("The container %1 does not exist in the directory"),
                      clusterCont
                    )
                  )
                  next if Ncs.testMode == false
                end
              end #existing cluster
            else
              # Validate existance of the cluster object in the directory
              realDN = GetDirectoryObjectsActualDN(
                adminDn,
                adminPassword,
                Ops.get_string(primary_server2, "CONFIG_LDAP_SERVER_IP", ""),
                ldapSecurePort,
                clusterFDN,
                "nCSNetWareCluster"
              )
              # strips leading and trailing junk characters
              realDN = NovellUtils.Strip(realDN, " \n \t")
              if realDN == ""
                if ldapSecurePort == 1636
                  realDN2 = GetDirectoryObjectsActualDN(
                    adminDn,
                    adminPassword,
                    Ops.get_string(primary_server2, "CONFIG_LDAP_SERVER_IP", ""),
                    636,
                    clusterFDN,
                    "nCSNetWareCluster"
                  )
                  # strips leading and trailing junk characters
                  realDN2 = NovellUtils.Strip(realDN2, " \n \t")

                  if realDN2 == ""
                    Report.Error(
                      Builtins.sformat(
                        _(
                          "The specified cluster object %1 could not be found in the directory"
                        ),
                        clusterFDN
                      )
                    )
                    next if Ncs.testMode == false
                  end
                else
                  Report.Error(
                    Builtins.sformat(
                      _(
                        "The specified cluster object %1 could not be found in the directory"
                      ),
                      clusterFDN
                    )
                  )
                  next if Ncs.testMode == false
                end
              end

              clusterFDN = realDN if realDN != ""

              sharedDisk = isSharedDiskCluster(
                adminDn,
                adminPassword,
                Ops.get_string(primary_server2, "CONFIG_LDAP_SERVER_IP", ""),
                ldapSecurePort,
                clusterFDN
              )
              next if @goForwardAnyway == false if sharedDisk
              if !sharedDisk # port 1636 does not work in all dsfw situations
                if ldapSecurePort == 1636
                  sharedDisk = isSharedDiskCluster(
                    adminDn,
                    adminPassword,
                    Ops.get_string(primary_server2, "CONFIG_LDAP_SERVER_IP", ""),
                    636,
                    clusterFDN
                  )
                  next if @goForwardAnyway == false if sharedDisk
                end
              end
              if sharedDisk
                SCR.Execute(path(".target.bash_output"), "modprobe nwraid")
                if !FindSBD(clusterFDN)
                  msg = "The install may complete without the SBD. However, the SBD\n" +
                    "partition is required for this node to join the cluster. It is\n" +
                    "recommended that you make sure this node can see the shared\n" +
                    "storage before proceeding.\n" +
                    "\nContinue anyway?"
                  if false ==
                      Popup.YesNoHeadline(
                        _("Failed to find SBD partition!"),
                        msg
                      )
                    next if Ncs.testMode == false
                  end
                end
              end
            end
          end

          # Make sure the IP address is valid
          next if rb == :new && !NovellValidators.ValidateIP(clusterIP)
          if rb == :new && !ValidateClusterIP(clusterIP)
            if false ==
                Popup.YesNo(
                  Builtins.sformat(
                    _(
                      "The address %1 is already in use. Continue and use anyway?"
                    ),
                    clusterIP
                  )
                )
              next if Ncs.testMode == false
            end
          end
          Ops.set(Ncs.Settings, "CONFIG_NCS_SBD_SIZE", deviceSize)
          if useMaxSize
            Ops.set(Ncs.Settings, "CONFIG_NCS_USE_MAX_SIZE", "yes")
          else
            Ops.set(Ncs.Settings, "CONFIG_NCS_USE_MAX_SIZE", "no")
          end
          Ops.set(Ncs.Settings, "CONFIG_NCS_LDAP_SERVERS", ldap_servers)
          Ops.set(
            Ncs.Settings,
            "CONFIG_NCS_CONFIG_TYPE",
            rb == :existing ? "Existing Cluster" : "New Cluster"
          )
          Ops.set(Ncs.Settings, "CONFIG_NCS_CLUSTER_DN", clusterFDN)
          if rb == :new
            Ops.set(Ncs.Settings, "CONFIG_NCS_CLUSTER_IP", clusterIP)
            Ops.set(Ncs.Settings, "CONFIG_NCS_SBD_DEV", device)
            Ops.set(Ncs.Settings, "CONFIG_NCS_SBD_DEV2", device2)
          end
          break
        elsif ret == :browse
          adminDn = Ops.get_string(
            NovellLDAP.Settings,
            "CONFIG_LDAP_ADMIN_CONTEXT",
            ""
          )
          adminPassword = Ops.get_string(
            NovellLDAP.Settings,
            "CONFIG_LDAP_ADMIN_PASSWORD",
            ""
          )
          if Ncs.xadMode
            xad_tree_admin_context = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_CONTEXT",
              ""
            )
            Builtins.y2milestone(
              "NovellLum:XAD_TREE_ADMIN_CONTEXT:%1",
              xad_tree_admin_context
            )
            if xad_tree_admin_context != nil && xad_tree_admin_context != ""
              adminDn = NovellUtils.Nds2Ldap(xad_tree_admin_context)
              adminPassword = Ops.get_string(
                NovellLDAP.Settings,
                "XAD_TREE_ADMIN_PASSWORD",
                ""
              )
            end
          end
          adminDn = NovellUtils.Nds2Ldap(adminDn)
          Builtins.y2milestone("Ncs:admin:%1", adminDn)
          wrkContainer = NovellLdapBrowser.LdapBrowser(
            Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", ""),
            636,
            true,
            adminDn,
            adminPassword,
            "",
            "",
            _("Browse for container")
          )
          Builtins.y2milestone(
            Builtins.sformat(
              "dialogs.ycp.NewOrExistingDialog:container from browse is: '%1'",
              wrkContainer
            )
          )
          if wrkContainer != nil && wrkContainer != ""
            wrkContainer = NovellUtils.Nds2Ldap(wrkContainer)
            if Builtins.substring(wrkContainer, 0, 3) != "cn="
              wrkContainer = Ops.add("cn=cluster,", wrkContainer)
            end
            UI.ChangeWidget(Id(:clusterFDN), :Value, wrkContainer)
          end
        elsif ret == :maxSize
          if Convert.to_boolean(UI.QueryWidget(Id(:maxSize), :Value))
            UI.ChangeWidget(Id(:deviceSize), :Enabled, false)
          else
            UI.ChangeWidget(Id(:deviceSize), :Enabled, true)
          end

          next
        elsif ret == :back
          break
        elsif ret == :new
          UI.ChangeWidget(Id(:clusterIP), :Enabled, true)
          UI.ChangeWidget(Id(:device), :Enabled, true)
          UI.ChangeWidget(Id(:device2), :Enabled, false)
          UI.ChangeWidget(Id(:clusterFDN), :Enabled, true)

          if Convert.to_boolean(UI.QueryWidget(Id(:maxSize), :Value))
            UI.ChangeWidget(Id(:deviceSize), :Enabled, false)
          else
            UI.ChangeWidget(Id(:deviceSize), :Enabled, true)
          end

          UI.ChangeWidget(Id(:maxSize), :Enabled, true)
          next
        elsif ret == :existing
          UI.ChangeWidget(Id(:clusterIP), :Enabled, false)
          UI.ChangeWidget(Id(:device), :Enabled, false)
          UI.ChangeWidget(Id(:device2), :Enabled, false)
          UI.ChangeWidget(Id(:clusterFDN), :Enabled, true)
          UI.ChangeWidget(Id(:deviceSize), :Enabled, false)
          UI.ChangeWidget(Id(:maxSize), :Enabled, false)
          next
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end
      deep_copy(ret)
    end

    # Proxy dialog
    # @return dialog result
    def ProxyDialog
      # Ncs Proxy dialog caption
      caption = _("OES Cluster Services (NCS) Proxy User Configuration")

      # Ncs Node dialog contents
      proxyUser = NovellUtils.Nds2Ldap(
        Ops.get_string(Ncs.Settings, "CONFIG_NCS_PROXY_USER", "")
      )
      proxyPassword = Ops.get_string(
        Ncs.Settings,
        "CONFIG_NCS_PROXY_PASSWORD",
        ""
      )

      use_common = Ops.get_string(
        Ncs.Settings,
        "CONFIG_NCS_USE_COMMON_PROXY",
        "yes"
      )
      doCommon = true
      doCommon = false if use_common != "yes"

      commonProxyName = NovellUtils.Nds2Ldap(
        Ops.get_string(Ncs.Settings, "CONFIG_NCS_COMMON_PROXY", "")
      )
      commonProxyPassword = Ops.get_string(
        Ncs.Settings,
        "CONFIG_NCS_COMMON_PROXY_PASSWORD",
        ""
      )

      adminDn = NovellUtils.Nds2Ldap(
        Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
      )
      adminPassword = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        ""
      )
      if Ncs.xadMode
        xad_tree_admin_context = Ops.get_string(
          NovellLDAP.Settings,
          "XAD_TREE_ADMIN_CONTEXT",
          ""
        )
        Builtins.y2milestone(
          "NovellLum:XAD_TREE_ADMIN_CONTEXT:%1",
          xad_tree_admin_context
        )
        if xad_tree_admin_context != nil && xad_tree_admin_context != ""
          adminDn = NovellUtils.Nds2Ldap(xad_tree_admin_context)
          adminPassword = Ops.get_string(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_PASSWORD",
            ""
          )
        end
      end
      Builtins.y2milestone("Ncs.ProxyDialog:admin:%1", adminDn)

      if proxyUser == adminDn && proxyPassword == ""
        proxyPassword = adminPassword
      end

      useCommonProxy = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_DO_PROXY",
        ""
      )

      contents = HVSquash(
        VBox(
          TextEntry(
            Id(:proxy_user),
            Opt(:notify),
            _("&NCS Proxy User Name (e.g. cn=admin.o=microfocus)"),
            proxyUser
          ),
          Left(
            Password(
              Id(:proxy_password),
              _("Specify NCS Proxy User &Password"),
              proxyPassword
            )
          ),
          Left(
            CheckBox(
              Id(:use_common),
              Opt(:notify),
              _("&Use OES Common Proxy User"),
              doCommon
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "Proxy", ""),
        Label.BackButton,
        Label.NextButton
      )
      ret = nil

      if useCommonProxy == "no"
        UI.ChangeWidget(Id(:use_common), :Enabled, false)
        UI.SetFocus(Id(:proxy_user))
      else
        UI.SetFocus(Id(:next))
        if proxyUser != "" && proxyUser == commonProxyName
          UI.ChangeWidget(Id(:proxy_user), :Enabled, false)
          UI.ChangeWidget(Id(:use_common), :Value, true)
          if proxyPassword != ""
            UI.ChangeWidget(Id(:proxy_password), :Enabled, false)
          end
        else
          UI.ChangeWidget(Id(:use_common), :Value, false)
          UI.ChangeWidget(Id(:proxy_password), :Enabled, true)
        end
      end

      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :next
          proxyUser = Convert.to_string(UI.QueryWidget(Id(:proxy_user), :Value))
          if proxyUser == nil || proxyUser == ""
            Popup.Error(
              _("Please enter a valid proxy user fully distinguished name.")
            )
            next
          end
          if !NovellValidators.ValidateFDN(proxyUser, false)
            UI.SetFocus(Id(:proxy_user))
            next
          end
          # Warning Message if user is configuring with service proxy
          if !(Convert.to_boolean(UI.QueryWidget(Id(:use_common), :Value)))
            Report.Warning(NovellUtils.removeServiceProxyMsg)
          end

          proxyPassword = Convert.to_string(
            UI.QueryWidget(Id(:proxy_password), :Value)
          )
          if !NovellValidators.ValidatePassword(proxyPassword)
            UI.SetFocus(Id(:proxy_password))
            next
          end

          replica_address = Ops.get_string(Ncs.Settings, "PRIMARY_IP", "")
          secure_port = Ops.get_integer(Ncs.Settings, "SECURE_LDAP", 636)

          # validate that the user exists by calling ValidateCredentials
          iVal = NovellValidators.ValidateCredentials(
            proxyUser,
            proxyPassword,
            replica_address,
            secure_port,
            true
          )
          Builtins.y2milestone(
            "Ncs:validating creds. name %1, pass ****, ip %2, port %3",
            proxyUser,
            replica_address,
            secure_port
          )
          if iVal != 0
            Builtins.y2milestone("ValidateCredentials() returned %1", iVal)
            if iVal == -666
              err = _(
                "The attempt to perform a secure LDAP bind did not respond in 2 minutes\n" +
                  "This may indicate a problem with LDAP on the target server.\n" +
                  "If you are confident that your credentials are correct you may continue,\n" +
                  "otherwise please determine if LDAP is running correctly on the target system\n" +
                  "Do you want to continue?"
              )
              yes = Popup.YesNoHeadline(_("LDAP secure bind timeout"), err)
              next if !yes
            else
              err = Builtins.sformat(
                _(
                  "Selected user must exist.  User credentials failed to validate Using:\n" +
                    "      Server: %1\n" +
                    "      User: %2\n" +
                    "      LDAP port: %3\n" +
                    "Do you want to continue anyway?"
                ),
                replica_address,
                proxyUser,
                secure_port
              )
              yes = Popup.YesNoHeadline(_("Credentials Failed"), err)
              next if !yes
            end
          end

          Ops.set(Ncs.Settings, "CONFIG_NCS_PROXY_USER", proxyUser)
          Ops.set(Ncs.Settings, "CONFIG_NCS_PROXY_PASSWORD", proxyPassword)
          break
        elsif ret == :proxy_user
          proxyUser = NovellUtils.Nds2Ldap(
            Convert.to_string(UI.QueryWidget(Id(:proxy_user), :Value))
          )
          if Builtins.tolower(proxyUser) ==
              NovellUtils.Nds2Ldap(Builtins.tolower(commonProxyName))
            UI.ChangeWidget(Id(:proxy_user), :Enabled, false)
            UI.ChangeWidget(Id(:use_common), :Value, true)
            if commonProxyPassword != ""
              UI.ChangeWidget(Id(:proxy_password), :Enabled, false)
            else
              UI.ChangeWidget(Id(:proxy_password), :Enabled, true)
            end
            UI.ChangeWidget(Id(:proxy_password), :Value, commonProxyPassword)
          else
            UI.ChangeWidget(Id(:proxy_user), :Enabled, true)
            UI.ChangeWidget(Id(:proxy_password), :Enabled, true)
          end

          next
        elsif ret == :use_common
          if Convert.to_boolean(UI.QueryWidget(Id(:use_common), :Value))
            UI.ChangeWidget(Id(:proxy_user), :Enabled, false)
            if commonProxyPassword != ""
              UI.ChangeWidget(Id(:proxy_password), :Enabled, false)
            end
            UI.ChangeWidget(Id(:proxy_user), :Value, commonProxyName)
            UI.ChangeWidget(Id(:proxy_password), :Value, commonProxyPassword)
          else
            UI.ChangeWidget(Id(:proxy_user), :Enabled, true)
            UI.ChangeWidget(Id(:proxy_password), :Enabled, true)
            UI.ChangeWidget(Id(:proxy_user), :Value, adminDn)
            UI.ChangeWidget(Id(:proxy_password), :Value, adminPassword)
          end
          next
        elsif ret == :back
          break
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end
      deep_copy(ret)
    end

    # Node dialog
    # @return dialog result
    def NodeDialog
      # Ncs Node dialog caption
      caption = _("OES Cluster Services (NCS) Configuration")

      # Ncs Node dialog contents
      node = Ops.get_string(Ncs.Settings, "CONFIG_NCS_SERVER_NAME", "")
      nodeIP = Ops.get_string(Ncs.Settings, "CONFIG_NCS_SERVER_IP", "")
      start = Ops.get_string(Ncs.Settings, "CONFIG_NCS_START", "Now")
      ipaddrs = Convert.convert(
        NovellUtils.GetAllStaticIPAddrs,
        :from => "list",
        :to   => "list <string>"
      )

      if Ncs.autoyast_tool
        ipaddrs = [_("Probed at install")]
        node = _("Local Host")
      else
        nodeIP = Ops.get(ipaddrs, 0, "") if !Builtins.contains(ipaddrs, nodeIP)
      end

      start = "Later" if Mode.installation && !Stage.normal

      contents = HVSquash(
        VBox(
          Left(Label(Builtins.sformat(_("Name of this node: %1"), node))),
          Left(ComboBox(Id(:nodeIP), _("IP &address of this node"), ipaddrs)),
          Left(
            CheckBox(
              Id(:start),
              _("&Start Clustering Services now"),
              start == "Now"
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "Node", ""),
        Label.BackButton,
        Label.FinishButton
      )
      ret = nil
      UI.ChangeWidget(Id(:nodeIP), :Value, nodeIP)
      if Mode.installation && !Stage.normal
        UI.ChangeWidget(Id(:start), :Enabled, false)
      end

      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :next
          nodeIP = Convert.to_string(UI.QueryWidget(Id(:nodeIP), :Value))

          #make sure we have a valid IP address
          if !Ncs.autoyast_tool
            if nodeIP == ""
              Popup.Error(
                _("The IP address / hostname of this node must be filled in.")
              )
              next if Ncs.testMode == false
            end
            if !NovellValidators.ValidateIP(nodeIP)
              next if Ncs.testMode == false
            end
          end

          # Check the now or later button
          b = Convert.to_boolean(UI.QueryWidget(Id(:start), :Value))
          if b
            start = "Now"
          else
            start = "Later"
          end

          Ops.set(Ncs.Settings, "CONFIG_NCS_SERVER_IP", nodeIP)
          Ops.set(Ncs.Settings, "CONFIG_NCS_START", start)
          break
        elsif ret == :back
          break
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end
      deep_copy(ret)
    end
  end
end
