# encoding: utf-8

# Copyright (c) 2020 Micro Focus or one of its affiliates.
# File:	include/novell-ncs/helps.ycp
# Package:	Configuration of Cluster Services (NCS)
# Summary:	Help texts of all the dialogs
# Authors:	Michael G. Fritch <mgfritch@novell.com>
#
# $Id: helps.ycp,v 1.1.2.8 2005/05/17 23:14:11 mclark Exp $
module Yast
  module NovellNcsHelpsInclude
    def initialize_novell_ncs_helps(include_target)
      textdomain "novell-ncs"

      # All helps are here
      @HELPS = {
        # Read dialog help 1/2
        "read"          => _(
          "<p><b><big>Initializing OES Cluster Services (NCS) Configuration</big></b><br>\nPlease wait...<br></p>\n"
        ) +
          # Read dialog help 2/2
          _(
            "<p><b><big>Aborting Initialization:</big></b><br>\nSafely stop the configuration utility by clicking <b>Abort</b> now.</p>\n"
          ),
        # Write dialog help 1/2
        "write"         => _(
          "<p><b><big>Saving OES Cluster Services (NCS) Configuration</big></b><br>\nPlease wait...<br></p>\n"
        ) +
          # Write dialog help 2/2
          _(
            "<p><b><big>Aborting Saving:</big></b><br>\n" +
              "Stop the save procedure by pressing <b>Abort</b>.\n" +
              "This opens an additional dialog that indicates whether it is safe to do so.</p>\n"
          ),
        # NewOrExisting dialog help 1/7
        "NewOrExisting" => _(
          "<p><b><big>New or Existing Cluster</big></b><br>\nChoose to either create a new cluster or to install OES Cluster Services on a server that you will add to an existing cluster.</p>\n"
        ) +
          # LDAP Servers dialog help 2/7
          _(
            "<p><b><big>Directory Server Address</big></b><br>\n" +
              "The IP addresses / hostnames shown are the LDAP servers available for this service to use. Select an LDAP server from the list. If you want an additional level of redundancy, select more than one IP address / hostname.</p>\n" +
              "<p>Ensure that the servers you select have a master replica or read/write replica of eDirectory. If you need to add another LDAP server to the list, add it using the LDAP Configuration for Open Enterprise Services dialog.</p>\n"
          ) +
          # NewOrExisting dialog help 3/7
          _(
            "<p><b><big>Cluster FDN</big></b><br>\n" +
              "Specify the fully distinguished name (FDN) of the cluster. Use the comma format illustrated in the example. Do not use dots.</p>\n" +
              "<p>If you are creating a new cluster, this is the name you will give the new cluster and the eDirectory context where the new cluster object will reside.</p>\n" +
              "<p>If you are adding a server to an existing cluster, this is the name and eDirectory context of the cluster that you are adding this server to. Click Browse to select an existing cluster object.</p>\n"
          ) +
          # NewOrExisting dialog help 4/7
          _(
            "<p><b><big>Cluster IP Address</big></b><br>\n" +
              "If you are creating a new cluster, specify a unique IP address / hostname for the cluster.</p>\n" +
              "<p>The cluster IP address / hostname is separate from the server IP address / hostname and is required to be on the same IP subnet as the other cluster servers.</p>\n"
          ) +
          # NewOrExisting dialog help 5/7
          _(
            "<p><b><big>Storage Device With Shared Media</big></b><br>\n" +
              "If you are creating a new cluster, enter the device where the SBD partition will be created.</p>\n" +
              "<p>If you have a shared disk system or SAN attached to your cluster servers, OES Cluster Services will create a small cluster partition on that shared disk system.</p>\n" +
              "<p>For 512 byte devices, you must have at least 20 MB of free space on one of the shared disk drives to create the cluster partition. For 4Kn devices, it must be at least 80 MB. If no free space is available, the shared disk drives cannot be used by OES Cluster Services.</p>\n" +
              "<p>If you do not have a shared disk system connected to your cluster servers, accept the default (none).</p>\n"
          ) +
          # NewOrExisting dialog help 6/7
          _("<p><b><big>Optional Device for Mirrored Partition</big></b><br>\n" +
          "If you want to mirror the SBD partition for greater fault tolerance, select the device where you want to mirror to.\n" + 
          "You can mirror the SBD partition only on devices having the same logical sector size.</p>" + 
          "<p>You can also mirror SBD partitions after installing OES Cluster Services.</p>\n"
          ) +
          # NewOrExisting dialog help 7/7
          _(
            "<p><b><big>Desired Partition Size of Shared Media</big></b><br>\nThe partition size in MB of the shared Media. For 512 byte devices, the value must be 8 MB or greater. For 4Kn devices, it must be 64 MB or greater.</p>\n"
          ),
        # Proxy dialog help 1/3
        "Proxy"         => _(
          "<p><b><big>NCS Proxy User Name</big></b><br>\nSpecify the typeful distinguished username of the NCS Proxy User for this server, or select the 'Use OES Common Proxy User' check box.</p>\n"
        ) +
          # Proxy dialog help 2/3
          _(
            "<p><b><big>Specify NCS Proxy User Password</big></b><br>\nSpecify the password for the username you provided in NCS Proxy User Name, or select the 'Use OES Common Proxy User' check box.</p>\n"
          ) +
          # Proxy dialog help 3/3
          _(
            "<p><b><big>Use OES Common Proxy User</big></b><br>\n" +
              "This option is enabled and selected by default if you have selected the 'Use Common Proxy User as default for OES Products' check box during the eDirectory configuration. The NCS Proxy User Name is automatically populated with the username of the common proxy user and the password field is disabled.</p>\n" +
              "<p>Deselect this check box if you want to specify a different username and password to use as the NCS Proxy User.</p>\n" +
              "<p>If you later deselect the 'Use Common Proxy User as default for OES Products' check box in eDirectory, then the NCS Proxy User name and password are automatically set to use the LDAP admin and password.</p>\n"
          ),
        # Node dialog help 1/3
        "Node"          => _(
          "<p><b><big>Name of this node</big></b><br>\nThis field is inactive and is for reference only.</p>\n"
        ) +
          # Node dialog help 2/3
          _(
            "<p><b><big>IP Address / hostname of this node</big></b><br>\nThis field contains the IP address / hostname of this node. If this server has multiple IP addresses, you can change the default address to another value if desired.</p>\n"
          ) +
          # Node dialog help 3/3
          _(
            "<p><b><big>Start Cluster Services now</big></b><br>\nSelect this box if you want clustering to start now. If you want clustering to start after rebooting, or if you want to manually start it later, deselect this box. </p>\n"
          )
      } 

      # EOF
    end
  end
end
