# encoding: utf-8

# Copyright (c) 2020 Micro Focus or one of its affiliates.
# File:	include/novell-ncs/wizards.ycp
# Package:	Configuration of Cluster Services (NCS)
# Summary:	Wizards definitions
# Authors:	Michael G. Fritch <mgfritch@novell.com>
#
# $Id: wizards.ycp,v 1.1.2.6 2005/01/10 18:33:45 sbunch Exp $
module Yast
  module NovellNcsWizardsInclude
    def initialize_novell_ncs_wizards(include_target)
      Yast.import "UI"

      textdomain "novell-ncs"

      Yast.import "Sequencer"
      Yast.import "Wizard"

      Yast.include include_target, "novell-ncs/complex.rb"
      Yast.include include_target, "novell-ncs/dialogs.rb"
    end

    # Main workflow of the ncs configuration
    # @return sequence result
    def MainSequence
      aliases = {
        "NewOrExisting" => lambda { NewOrExistingDialog() },
        "Proxy"         => lambda { ProxyDialog() },
        "Node"          => lambda { NodeDialog() }
      }

      sequence = {
        "ws_start"      => "NewOrExisting",
        "NewOrExisting" => {
          :abort => :abort,
          :next  => "Proxy",
          :skip  => :next
        },
        "Proxy"         => { :abort => :abort, :next => "Node" },
        "Node"          => { :abort => :abort, :next => :next }
      }

      Sequencer.Run(aliases, sequence)
    end


    # Whole configuration of ncs
    # @return sequence result
    def NcsSequence
      aliases = {
        "read"  => [lambda { ReadDialog() }, true],
        "main"  => lambda { MainSequence() },
        "write" => [lambda { WriteDialog() }, true]
      }

      sequence = {
        "ws_start" => "read",
        "read"     => { :abort => :abort, :next => "main" },
        "main"     => { :abort => :abort, :next => "write" },
        "write"    => { :abort => :abort, :next => :next }
      }

      Wizard.CreateDialog
      Wizard.SetDesktopIcon("ncs")

      ret = Sequencer.Run(aliases, sequence)

      UI.CloseDialog
      deep_copy(ret)
    end

    # Whole configuration of ncs but without reading and writing.
    # For use with autoinstallation.
    # @return sequence result
    def NcsAutoSequence
      # Initialization dialog caption
      caption = _("OES Cluster Services (NCS) Configuration")
      # Initialization dialog contents
      contents = Label(_("Initializing..."))

      Wizard.CreateDialog
      Wizard.SetDesktopIcon("ncs")
      Wizard.SetContentsButtons(
        caption,
        contents,
        "",
        Label.BackButton,
        Label.NextButton
      )

      ret = MainSequence()

      UI.CloseDialog
      deep_copy(ret)
    end
  end
end
